import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;
import lab.game.*;
import java.io.IOException;

public class Main{
    public static void main(String[] args) throws IOException {
        Splash newSplash = new Splash();
        newSplash.run();
    }
    public static void rerun() throws IOException {
        Splash newSplash = new Splash();
        newSplash.run();
    }
}

class Splash{

    public void run() throws IOException {
        Object[] options = {"Server", "Client"};
        int result = JOptionPane.showOptionDialog(null, "Which role do you take?", "Start a Game", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[1]);
        if( result == JOptionPane.YES_OPTION ){ // Server

            ServerSocket server = new ServerSocket(5000);
            System.out.println("Server Created");

            JFrame frame = new JFrame();
            JPanel panel = new JPanel(new BorderLayout());
            JLabel label = new JLabel("Waiting for opponent...");
            label.setBorder(new EmptyBorder(40,40,40,40));
            Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
            panel.add(label);
            frame.add(label);
            frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            frame.pack();
            frame.setLocation(dim.width/2-frame.getSize().width/2, dim.height/2-frame.getSize().height/2);
            frame.setVisible(true);

            Socket socket = server.accept();
            System.out.println("Client Connected");
            frame.dispatchEvent(new WindowEvent(frame, WindowEvent.WINDOW_CLOSING));

            server.close();

            Chess newChess = new Chess("W", socket);
            newChess.run();
        }else { // Client
            String s = (String)JOptionPane.showInputDialog(
                    null,
                    "Enter Server IP:",
                    "Customized Dialog",
                    JOptionPane.PLAIN_MESSAGE,
                    null,
                    null,
                    "x.x.x.x");

            Socket socket = new Socket(s, 5000);
            System.out.println("Connected to server");

            Chess newChess = new Chess("B", socket);
            newChess.run();
        }
    }
}
/**
 * A New Button class that has piece attribute
 * @author Omiid
 */
class newJButton extends JButton{
    private Piece piece;
    private char X;
    private int Y;

    public newJButton(){
        this.piece = null;
        this.setIcon(null);
    }

    /**
     * Make new button
     * @param X The X-Axis
     * @param Y The Y-Axis
     * @author Omiid
     */
    public newJButton(char X, int Y){
        this.piece = null;
        this.X = X;
        this.Y = Y;
        this.setIcon(null);
    }

    /**
     * Make new button
     * @param p The piece
     * @param X The X-Axis
     * @param Y The Y-Axis
     * @author Omiid
     */
    public newJButton(Piece p, char X, int Y){
        this.piece = p;
        this.X = X;
        this.Y = Y;
        Image img = null;
        try {
            img = ImageIO.read(Chess.class.getResource("resources/images/" + p.getName().toLowerCase() + ".png"));
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        this.setIcon(new ImageIcon(img));
    }

    public Piece getPiece(){
        return this.piece;
    }


    public void setPiece(Piece p){
        this.piece = p;
    }

    public char getNewX(){
        return this.X;
    }

    public int getNewY(){
        return this.Y;
    }

}

/**
 * Main Chess Class
 */
class Chess implements MouseListener{
    private Socket socket = null;
    private DataInputStream input   = null;
    private DataOutputStream output = null;
    private Board board = null;
    private boolean me;
    private String color = "W";
    private boolean isSelected = false;
    private newJButton inMove = null;
    private JFrame frame;
    private ArrayList<newJButton> btns = new ArrayList<newJButton>();
    private ArrayList<newJButton> whiteLost = new ArrayList<>();
    private ArrayList<newJButton> blackLost = new ArrayList<>();
    private JLabel caption;

    public Chess(String me, Socket socket) throws IOException {
        if( me.equals("W") ){

            this.me = true;
        }else{
            this.me = false;
        }
        this.socket = socket;
        this.input  = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
        this.output = new DataOutputStream(this.socket.getOutputStream());
    }
    /**
     * Initial Run
     */
    public void run() throws IOException {
        JFrame frame = new JFrame("Chess Frame");
        frame.setExtendedState(java.awt.Frame.MAXIMIZED_BOTH);
        // Biggest Panel
        JPanel fullPanel = new JPanel(new BorderLayout());
        // Board Panel
        JPanel boardPanel = new JPanel(new GridLayout(8,8));
        ArrayList<Piece> whitePieces = new ArrayList<Piece>();
        ArrayList<Piece> blackPieces = new ArrayList<Piece>();
        if( this.me ){

            for(int i = 1; i < 9; i++){
                for(char j = 'A'; j < 'I'; j++){
                    newJButton btn = null;

                    if( i == 8 ){
                        if( j == 'A' || j == 'H' ){

                            Rook newRook = new Rook(j, i, true, "WR");
                            whitePieces.add(newRook);
                            btn = new newJButton(newRook, j, i);
                        }else if( j == 'B' || j == 'G' ){

                            Knight newKnight = new Knight(j, i, true, "WN");
                            whitePieces.add(newKnight);
                            btn = new newJButton(newKnight, j, i);
                        }else if( j == 'C' || j == 'F' ){

                            Bishop newBishop = new Bishop(j, i, true, "WB");
                            whitePieces.add(newBishop);
                            btn = new newJButton(newBishop, j, i);
                        }else if( j == 'D' ){

                            Queen whiteQueen = new Queen(j, i, true,"WQ");
                            whitePieces.add(whiteQueen);
                            btn = new newJButton(whiteQueen, j, i);
                        }else if( j == 'E' ){
                            King whiteKing = new King(j, i, true, "WK");
                            whitePieces.add(whiteKing);
                            btn = new newJButton(whiteKing, j, i);
                        }
                    }else if( i == 7 ){
                        Pawn newPawn = new Pawn(j, i, true, "WP");
                        whitePieces.add(newPawn);
                        btn = new newJButton(newPawn, j, i);
                    }else if( i == 1 ){
                        if( j == 'A' || j == 'H' ){

                            Rook newRook = new Rook(j, i, false, "BR");
                            blackPieces.add(newRook);
                            btn = new newJButton(newRook, j, i);
                        }else if( j == 'B' || j == 'G' ){

                            Knight newKnight = new Knight(j, i, false, "BN");
                            blackPieces.add(newKnight);
                            btn = new newJButton(newKnight, j, i);
                        }else if( j == 'C' || j == 'F' ){

                            Bishop newBishop = new Bishop(j, i, false, "BB");
                            blackPieces.add(newBishop);
                            btn = new newJButton(newBishop, j, i);
                        }else if( j == 'D' ){

                            Queen blackQueen = new Queen(j, i, false,"BQ");
                            blackPieces.add(blackQueen);
                            btn = new newJButton(blackQueen, j, i);
                        }else if( j == 'E' ){
                            King blackKing = new King(j, i, false, "BQ");
                            blackPieces.add(blackKing);
                            btn = new newJButton(blackKing, j, i);
                        }
                    }else if( i == 2 ){
                        Pawn newPawn = new Pawn(j, i, false, "BP");
                        blackPieces.add(newPawn);
                        btn = new newJButton(newPawn, j, i);
//                    btn = new newJButton(j, i);
                    }else{
                        btn = new newJButton(j, i);
                    }
                    this.board = new Board(whitePieces, blackPieces);
                    this.btns.add(btn);
                    btn.addMouseListener(this);
                    btn.setFocusable(false);
                    if( (i % 2 == 1 && j % 2 == 1) || (i % 2 == 0 && j % 2 == 0) ){
                        btn.setBackground(new Color(219,217,164));
                    }else{
                        btn.setBackground(new Color(46, 83,106));
                    }
                    boardPanel.add(btn);
                }
            }
        }else{
            for(int i = 8; i >= 1; i--){
                for(char j = 'H'; j >= 'A'; j--){
                    newJButton btn = null;

                    if( i == 8 ){
                        if( j == 'A' || j == 'H' ){

                            Rook newRook = new Rook(j, i, true, "WR");
                            whitePieces.add(newRook);
                            btn = new newJButton(newRook, j, i);
                        }else if( j == 'B' || j == 'G' ){

                            Knight newKnight = new Knight(j, i, true, "WN");
                            whitePieces.add(newKnight);
                            btn = new newJButton(newKnight, j, i);
                        }else if( j == 'C' || j == 'F' ){

                            Bishop newBishop = new Bishop(j, i, true, "WB");
                            whitePieces.add(newBishop);
                            btn = new newJButton(newBishop, j, i);
                        }else if( j == 'D' ){

                            Queen whiteQueen = new Queen(j, i, true,"WQ");
                            whitePieces.add(whiteQueen);
                            btn = new newJButton(whiteQueen, j, i);
                        }else if( j == 'E' ){
                            King whiteKing = new King(j, i, true, "WK");
                            whitePieces.add(whiteKing);
                            btn = new newJButton(whiteKing, j, i);
                        }
                    }else if( i == 7 ){
                        Pawn newPawn = new Pawn(j, i, true, "WP");
                        whitePieces.add(newPawn);
                        btn = new newJButton(newPawn, j, i);
                    }else if( i == 1 ){
                        if( j == 'A' || j == 'H' ){

                            Rook newRook = new Rook(j, i, false, "BR");
                            blackPieces.add(newRook);
                            btn = new newJButton(newRook, j, i);
                        }else if( j == 'B' || j == 'G' ){

                            Knight newKnight = new Knight(j, i, false, "BN");
                            blackPieces.add(newKnight);
                            btn = new newJButton(newKnight, j, i);
                        }else if( j == 'C' || j == 'F' ){

                            Bishop newBishop = new Bishop(j, i, false, "BB");
                            blackPieces.add(newBishop);
                            btn = new newJButton(newBishop, j, i);
                        }else if( j == 'D' ){

                            Queen blackQueen = new Queen(j, i, false,"BQ");
                            blackPieces.add(blackQueen);
                            btn = new newJButton(blackQueen, j, i);
                        }else if( j == 'E' ){
                            King blackKing = new King(j, i, false, "BQ");
                            blackPieces.add(blackKing);
                            btn = new newJButton(blackKing, j, i);
                        }
                    }else if( i == 2 ){
                        Pawn newPawn = new Pawn(j, i, false, "BP");
                        blackPieces.add(newPawn);
                        btn = new newJButton(newPawn, j, i);
//                    btn = new newJButton(j, i);
                    }else{
                        btn = new newJButton(j, i);
                    }
                    this.board = new Board(whitePieces, blackPieces);
                    this.btns.add(btn);
                    btn.addMouseListener(this);
                    btn.setFocusable(false);
                    if( (i % 2 == 1 && j % 2 == 1) || (i % 2 == 0 && j % 2 == 0) ){
                        btn.setBackground(new Color(219,217,164));
                    }else{
                        btn.setBackground(new Color(46, 83,106));
                    }
                    boardPanel.add(btn);
                }
            }
        }

        // Left Divider Panel
        JPanel leftDivider = new JPanel(new GridLayout(3, 1));
        // Top Left
        JPanel topLeft = new JPanel(new GridLayout(2, 8));
        topLeft.setBorder(new EmptyBorder(50, 10, 50, 0));
        for(char i = 0; i < 2; i++){
            for(int j = 0; j < 8; j++){
                newJButton btn = new newJButton();
                ImageIcon icon = new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB));
                btn.setIcon(icon);
                btn.setPreferredSize(new Dimension(70, 70));
                btn.setFocusable(false);
                btn.setBackground(new Color(46, 83,106));
                topLeft.add(btn);
                whiteLost.add(btn);
            }
        }
        // Middle Left
        JPanel middleLeft = new JPanel();
        middleLeft.setBorder(new EmptyBorder(100, 0, 0, 0));
        JLabel playing = new JLabel("White Player is playing");
        playing.setFont(new Font("Segoe UI", Font.BOLD, 32));
        caption = playing;
        middleLeft.add(playing);
        leftDivider.add(middleLeft);
        // Bottom Left
        JPanel bottomLeft = new JPanel(new GridLayout(2, 8));
        bottomLeft.setBorder(new EmptyBorder(50, 10, 50, 0));
        for(int i = 0; i < 2; i++){
            for(int j = 0; j < 8; j++){
                newJButton btn = new newJButton();
                ImageIcon icon = new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB));
                btn.setPreferredSize(new Dimension(70, 70));
                btn.setIcon(icon);
                btn.setFocusable(false);
                btn.setBackground(new Color(219,217,164));
                bottomLeft.add(btn);
                blackLost.add(btn);
            }
        }
        // Add Panels
        leftDivider.add(topLeft);
        leftDivider.add(middleLeft);
        leftDivider.add(bottomLeft);

        fullPanel.add(boardPanel, BorderLayout.EAST);
        fullPanel.add(leftDivider, BorderLayout.WEST);

        frame.add(fullPanel);
        // Show Frame
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);

        this.frame = frame;

        String line = "";
        while (true)
        {
            try
            {
                line = this.input.readUTF();
                if( line.equals("Over") ){
                    break;
                }
                System.out.println(line);
                String[] splited = line.split(" ");
                for(newJButton btn : this.btns){
                    if( btn.getNewX() == splited[0].charAt(1) && btn.getNewY() == splited[0].charAt(0) - '0' ){

                        System.out.println("Found inmove");
                        this.inMove = btn;
                    }
                }
                newJButton source = null;
                for(newJButton btn : this.btns){
                    if( btn.getNewX() == splited[1].charAt(1) && btn.getNewY() == splited[1].charAt(0) - '0' ){

                        System.out.println("Found source");
                        source = btn;
                    }
                }
                if( this.board.canGo(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){
                    if( this.board.canAttack(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){

                        Piece temp = source.getPiece();
                        source.setPiece(this.inMove.getPiece());
                        this.inMove.setPiece(null);
                        if( this.color.equals("W") ){

                            killBlack(temp);
                        }else{
                            killWhite(temp);
                        }
                    }else{
                        Piece temp = this.inMove.getPiece();
                        this.inMove.setPiece(source.getPiece());
                        source.setPiece(temp);
                    }
                    System.out.println(this.board.move( inMove.getNewY() + Character.toString(inMove.getNewX()), source.getNewY() + Character.toString(source.getNewX()), this.color));
                    if( this.color.equals("W")){

                        caption.setText("Black Player is playing");
                        this.color = "B";
                    }else{
                        caption.setText("White Player is playing");
                        this.color = "W";
                    }
                    this.isSelected = false;
                    this.inMove = null;
                }
                repaint();
                try {
                    checkMate();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            catch(IOException i)
            {
                System.out.println(i);
            }
        }
        if( this.output != null ){

            try {
                this.output.writeUTF("Over");
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
        System.out.println("Closing connection");

        this.socket.close();
        this.input.close();
        this.output.close();
}

    /**
     * Mouse Clicking behaviour
     * @param e The event Happened
     */
    @Override
    public void mouseClicked(MouseEvent e) {
        newJButton source = (newJButton)(e.getSource());
        if( !this.isSelected ){
            boolean flag = false;
            if( this.me && this.color.equals("W") ){

                flag = true;
            }else if( !this.me && this.color.equals("B") ){
                flag = true;
            }
            if( source.getPiece() != null && source.getPiece().getColor() == this.me && flag ){

                for(newJButton btn : this.btns){
                    if( source != btn && source.getPiece() != null && this.board.canGo(source.getPiece(), btn.getNewX(), btn.getNewY(), this.color) ){
                        if( this.board.canAttack(source.getPiece(), btn.getNewX(), btn.getNewY(), this.color) ){

                            btn.setBackground(new Color(255, 0, 0));
                        }else{
                            Image img = null;
                            try {
                                img = ImageIO.read(Chess.class.getResource("resources/images/dot.png"));
                            } catch (IOException e1) {
                                e1.printStackTrace();
                            }
                            btn.setIcon(new ImageIcon(img, "dot"));
                        }
                        this.isSelected = true;
                        this.inMove = source;
                    }
                }
            }
        }else{
            if( this.board.canGo(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){
                if( this.board.canAttack(this.inMove.getPiece(), source.getNewX(), source.getNewY(), this.color) ){

                    Piece temp = source.getPiece();
                    source.setPiece(this.inMove.getPiece());
                    this.inMove.setPiece(null);
                    if( this.color.equals("W") ){

                        killBlack(temp);
                    }else{
                        killWhite(temp);
                    }
                }else{
                    Piece temp = this.inMove.getPiece();
                    this.inMove.setPiece(source.getPiece());
                    source.setPiece(temp);
                }
                if( this.board.move( inMove.getNewY() + Character.toString(inMove.getNewX()), source.getNewY() + Character.toString(source.getNewX()), this.color) ){
                    try {
                        this.output.writeUTF(inMove.getNewY() + Character.toString(inMove.getNewX()) + " " + source.getNewY() + source.getNewX());
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
                    System.out.println(true);
                }
                if( this.color.equals("W")){

                    caption.setText("Black Player is playing");
                    this.color = "B";
                }else{
                    caption.setText("White Player is playing");
                    this.color = "W";
                }
            }
            this.isSelected = false;
            this.inMove = null;
            repaint();
            try {
                checkMate();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }

    public void repaint(){
        for(newJButton btn : this.btns) {
            if( btn.getPiece() != null ){
                Image img = null;
                try {
                    img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
                btn.setIcon(new ImageIcon(img));
            }else{
                btn.setIcon(null);
            }
            if( (btn.getNewY() % 2 == 1 && btn.getNewX() % 2 == 1) || (btn.getNewY() % 2 == 0 && btn.getNewX() % 2 == 0) ){
                btn.setBackground(new Color(219,217,164));
            }else{
                btn.setBackground(new Color(46, 83,106));
            }
        }
        for(newJButton btn : this.whiteLost) {
            if( btn.getPiece() != null ){
                Image img = new BufferedImage(50, 50, BufferedImage.TYPE_INT_ARGB);
                try {
                    img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
                btn.setIcon(new ImageIcon(img));
            }else{
                btn.setIcon(new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB)));
            }
        }
        for(newJButton btn : this.blackLost) {
            if( btn.getPiece() != null ){
                Image img = new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB);
                try {
                    img = ImageIO.read(Chess.class.getResource("resources/images/" + btn.getPiece().getName().toLowerCase() + ".png"));
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
                btn.setIcon(new ImageIcon(img));
            }else{
                btn.setIcon(new ImageIcon(
                        new BufferedImage(40, 40, BufferedImage.TYPE_INT_ARGB)));
            }
        }
    }

    public void checkMate() throws IOException {
        if( this.board.checkMate() != null ){

            if ( !this.socket.isClosed() ){

                try {
                    this.output.writeUTF("Over");
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            String prompt = "";
            if( this.board.checkMate().equals("W") ){

                prompt = prompt.concat("Black Player Won!");
            }else{
                prompt = prompt.concat("White Player Won!");
            }
            prompt = prompt.concat(" \nDo you want to start a new game?");
            this.frame.dispatchEvent(new WindowEvent(this.frame, WindowEvent.WINDOW_CLOSING));
            int result = JOptionPane.showConfirmDialog(this.frame, prompt, "Finished", JOptionPane.YES_NO_OPTION);
            if( result == JOptionPane.YES_OPTION ){

                Main.rerun();
            }else{
                System.exit(0);
            }
        }
    }

    public boolean killWhite(Piece p){
        for(newJButton btn : this.whiteLost){
            if( btn.getPiece() == null ){

                btn.setPiece(p);
                return true;
            }
        }
        return false;
    }

    public boolean killBlack(Piece p){
        for(newJButton btn : this.blackLost){
            if( btn.getPiece() == null ){

                btn.setPiece(p);
                return true;
            }
        }
        return false;
    }

    @Override
    public void mousePressed(MouseEvent e) {

    }

    @Override
    public void mouseReleased(MouseEvent e) {

    }

    @Override
    public void mouseEntered(MouseEvent e) {

    }

    @Override
    public void mouseExited(MouseEvent e) {

    }
}
